#!/usr/bin/python -tt
#=======================================================================
#                        General Documentation

"""Single-class module.

   See class docstring for description.
"""

#-----------------------------------------------------------------------
#                       Additional Documentation
#
# RCS Revision Code:
#   $Id: atmconst.py,v 1.1 2004/02/19 22:28:12 jlin Exp $
#
# Modification History:
# - 13 Oct 2003:  Original by Johnny Lin, Computation Institute,
#   University of Chicago.  Email:  air_jlin@yahoo.com.  Passed
#   passably reasonable tests.
# - 17 Jan 2004:  Changes to g.
#
# Notes:
# - Written for Python 2.2.
# - No non-built-in packages and modules required.
#
# Copyright (c) 2003 by Johnny Lin.  For licensing, distribution 
# conditions, contact information, and additional documentation see
# the URL http://www.johnny-lin.com/py_pkgs/atmqty/.
#=======================================================================




#---------------- Module General Import and Declarations ---------------

#- Set module version number to package version number:

import package_version
__version__ = package_version.version
del package_version




#------------- Class AtmConst:  All Attributes and Methods -------------

class AtmConst:
    """Key atmospheric-related constants, defined for the earth.

    Class attributes:
    * epsilon:  Ratio of gas constant for dry air to gas constant 
      for water vapor [unitless].
    * g:  Gravitational constant [m/s**2].  Set to g_wmo.
    * g_approx:  Approximate value of gravitational constant [m/s**2].
      This value is commonly used in rough calculations.
    * g_rean:  Gravitational constant [m/s**2] used in most calcula-
      tions in the NCEP/NCAR reanalysis.  See the FAQ for details:
      http://dss.ucar.edu/pub/reanalysis/FAQ.html.
    * g_wmo:  Gravitational constant [m/s**2] used by the WMO to cali-
      brate barometers.
    * Gamma_d:  Dry adiabatic lapse rate [K/m].
    * R_d:  Gas constant for dry air [J/(K kg)].
    * R_univ:  Universal gas constant [J/(mol kg)].
    * R_v:  Gas constant for water vapor [J/(K kg)].
    * c_p:  Specific heat at constant pressure for dry air [J/(K kg)].
    * C2K_add_offset:  Value to add to degrees Centigrade to convert 
      to Kelvin.  0 deg C is defined to be at the ice point, which is
      273.15 K.  Note the triple point and ice point are different.
      In Kelvin, the fundamental reference point is the triple point,
      which equals 273.16.  See Weast (p. F-73, 90) for details.
    * ice_point:  The ice point of water [K].
    * kappa:  Ratio R_d to c_p [unitless].
    * triple_point:  The triple point of water [K].

    This class accepts no arguments and has no methods.

    References:
    * Glickman, T. S. (Ed.) (2000):  Glossary of Meterology.  Boston,
      MA:  American Meteorology Society, 855 pp.
    * Peixoto, J. P., and A. H. Oort (1992):  Physics of Climate.  
      New York, NY:  American Institute of Physics, ISBN 0-88318-711-6, 
      520 pp.
    * Shea, D. J., S. J. Worley, I. R. Stern, and T. J. Hoar (1995, 
      2003): An Introduction to Atmospheric and Oceanographic Datasets, 
      NCAR Technical Note NCAR/TN-404+IA.  Boulder, CO:  NCAR.  Web 
      copy: http://www.cgd.ucar.edu/cas/tn404/.

    * Weast, R. C. (Ed.) (1987):  CRC Handbook of Chemistry and Physics
      (68th Edition).  Boca Raton, FL:  CRC Press, Inc.
    * Wallace, J. M., and P. V. Hobbs (1977): Atmospheric Science:  
      An Introductory Survey.  San Diego, CA:  Academic Press, ISBN 
      0-12-732950-1, 467 pp.

    Examples:
    >>> from atmconst import AtmConst
    >>> const = AtmConst()
    >>> const.R_d
    287.05000000000001
    >>> const.g
    9.8066499999999994
    """
    def __init__(self):

        #- Non-derived constants:
        self.c_p = 1004.              # (Wallace & Hobbs, p. 69)
        self.g_approx = 9.81          # (Wallace & Hobbs, p. 68)
        self.g_rean = 9.8
        self.g_wmo = 9.80665          # (Shea et al., Ch. 3)
        self.ice_point = 273.15       # (Weast, p. F-90)
        self.R_d = 287.05             # (Garratt, p. 283)
        self.R_univ = 8.316963        # (Glickman, p. 328)
        self.R_v = 461.53             # (Garratt, p. 283)
        self.triple_point = 273.16    # (Weast, p. F-90)


        #- Derived constants:
        self.C2K_add_offset = self.ice_point
        self.epsilon = self.R_d / self.R_v
        self.g = self.g_wmo
        self.Gamma_d = self.g / self.c_p
        self.kappa = self.R_d / self.c_p




#-------------------------- Main:  Test Module -------------------------

#- Define additional examples for doctest to use:

__test__ = { 
    'Additional Example 1':
    """
    >>> from atmconst import AtmConst
    >>> const = AtmConst()
    >>> const.kappa
    0.28590637450199202
    >>> const.epsilon
    0.62195306913960091
    >>> const.g_approx
    9.8100000000000005
    >>> const.g_rean
    9.8000000000000007
    >>> const.g_wmo
    9.8066499999999994
    """
    }


#- Execute doctest if module is run from command line:

if __name__ == "__main__":
    """Test the module.

    Tests the examples in all the module documentation strings, plus 
    __test__.

    Note:  On at least 2 platforms the doctest module does not execute
    the commands in $HOME/.pythonrc.py when "python atmconst.py" is
    executed at the command line.  Since the package that contains this
    module might not be included on the default Python path, specifying 
    it in .pythonrc.py is one way to insure Python can access the 
    package in an interactive session.  If doctest does not execute 
    .pythonrc.py, any call to the package will fail.
    
    To ensure that module testing of this atmconst.py file works, the 
    parent directory to the current directory is added to sys.path.
    """
    import doctest, sys, os
    sys.path.append(os.pardir)
    doctest.testmod(sys.modules[__name__])




# ===== end file =====
