#!/usr/bin/python -tt
#=======================================================================
#                        General Documentation

"""Single-function module.

   See function docstring for description.
"""

#-----------------------------------------------------------------------
#                       Additional Documentation
#
# RCS Revision Code:
#   $Id: mix_ratio.py,v 1.1 2004/02/19 22:28:12 jlin Exp $
#
# Modification History:
# - 20 Oct 2003:  Original by Johnny Lin, Computation Institute,
#   University of Chicago.  Email:  air_jlin@yahoo.com.  Passed
#   passably reasonable tests.@@@need to check the eqns. work and
#   are correct in the derivation.
#
# Notes:
# - Written for Python 2.2.
# - Module docstrings can be tested using the doctest module.  To
#   test, execute "python mix_ratio.py".
# - Non-built-in packages and modules required:  atmqty
#
# Copyright (c) 2003 by Johnny Lin.  For licensing, distribution 
# conditions, contact information, and additional documentation see
# the URL http://www.johnny-lin.com/py_pkgs/atmqty/.
#=======================================================================




#---------------- Module General Import and Declarations ---------------

#- Set module version number to package version number:

import package_version
__version__ = package_version.version
del package_version




#-------------------- General Function:  mix_ratio ---------------------

def mix_ratio(variables):
    """Calculate mixing ratio over liquid water.

    Method Arguments:
    * variables:  A dictionary of input state variables, where for
      each key:value pair the key labels what state variable it is
      and the value is the state variable.  The value is a Numeric
      floating point array of any number of dimensions and size.
      All state variables must be arrays of the same shape and size.
      Argument variables can have any number of items.  Key values 
      may be any value, but values that may be used by this function 
      include:
      + 'e':  Vapor pressure over water [hPa].
      + 'p':  Total pressure [hPa].
      + 'q':  Specific humidity [kg/kg].

    Output:
    * Mixing ratio [kg/kg].  Numeric array of same dimensions and 
      size as state variables in the argument.

    The mixing ratio can be calculated from a variety of different
    combinations of state variables.  In this function, the following
    combinations are supported:
    * Vapor pressure and total pressure.
    * Specific humidity.
    The function will automatically apply the correct equation
    depending on the key values of the argument dictionary.  An
    error is returned if none of the above combinations are present
    as the input argument.

    Reference:
    * Emanuel, K. A. (1994):  Atmospheric Convection.  New York, NY:
      Oxford University Press, 580 pp.

    Example:
    >>> from mix_ratio import mix_ratio
    >>> import Numeric as N
    >>> p = N.array([1026.8])
    >>> e = N.array([9.00051379])
    >>> r = mix_ratio({'p':p, 'e':e})
    >>> r 
    array([5.5e-3])
    """

    #- Import statements:

    import atmqty
    import Numeric as N
    from atmqty.atmconst import AtmConst
    from atmqty.is_numeric_float import is_numeric_float


    #- Atmospheric constants:

    const = AtmConst()


    #- Check input is of the correct type:

    for akey in variables.keys():
        if is_numeric_float(variables[akey]) != 1:
            raise TypeError, "mix_ratio:  Arg not Numeric floating"


    #- Calculate the mixing ratio depending on what the inputs are
    #  (derived from Emanuel eqn. 4.1.2):

    if   variables.has_key('q') == 1:
        mix = variables['q'] / (1.0 - variables['q'])
    elif (variables.has_key('p') == 1) and \
         (variables.has_key('e') == 1):
        mix = const.epsilon * variables['e'] \
            / (variables['p'] - variables['e'])
    else:
        raise ValueError, "mix_ratio:  Bad input keys"


    #- Return mixing ratio:

    return mix



#-------------------------- Main:  Test Module -------------------------

#- Define additional examples for doctest to use:

__test__ = { 'Additional Example 1':
    """
    """ }


#- Execute doctest if module is run from command line:

if __name__ == "__main__":
    """Test the module.

    Tests the examples in all the module documentation strings, plus 
    __test__.

    Note:  On at least 2 platforms the doctest module does not execute
    the commands in $HOME/.pythonrc.py when "python mix_ratio.py" is
    executed at the command line.  Since atmqty might not be included
    on the default Python path, specifying it in .pythonrc.py is one
    way to insure Python can access the package in an interactive
    session.  If doctest does not execute .pythonrc.py, any call to 
    atmqty will fail.
    
    As the mix_ratio module requires the atmqty package (in order to 
    access the AtmConst class) and mix_ratio is distributed as part 
    of the atmqty package, the parent directory to the atmqty direc-
    tory is added to sys.path for this module testing case to enable 
    doctest to work on mix_ratio.
    """
    import doctest, sys, os
    sys.path.append(os.pardir)
    doctest.testmod(sys.modules[__name__])




# ===== end file =====
